import { Connection, PublicKey } from '@solana/web3.js';
import { Jupiter } from '@jup-ag/core';
import { LiquidityPoolVenues, TokenInfo, Routes, ZapTransactionsAndMeta } from './types';
/**
 * Zap class for swapping SPL tokens to LP tokens through rebalancing and staking.
 */
export declare class Zap {
    /**
     * Web3 connection instance.
     */
    connection: Connection;
    /**
     * Jupiter instance.
     */
    jupiter: Jupiter;
    /**
     * Cached token list.
     */
    allTokens: TokenInfo[];
    /**
     * Builds and returns a ZAP instance.
     */
    private constructor();
    /**
     * Initializes a ZAP instance.
     */
    static build(connection: Connection, user: PublicKey): Promise<Zap>;
    /**
     * Swap (zap) a single SPL token to an LP token by rebalancing and staking.
     * Helper method to bundling `swapToBaseQuote` and `addLiquidity` together.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @param user - user account
     * @param liquidityPoolVenue - liquidity pool venue
     * @param slippageTolerancePercent - slippage tolerance in percent, default is 1%
     * @returns Array of transactions
     */
    swapAndAddLiquidity({ inputTokenAmount, inputTokenMint, lpTokenMint, user, liquidityPoolVenue, slippageTolerancePercent }: {
        inputTokenAmount: number;
        inputTokenMint: PublicKey;
        lpTokenMint: PublicKey;
        user: PublicKey;
        liquidityPoolVenue?: LiquidityPoolVenues;
        slippageTolerancePercent?: number;
    }): Promise<ZapTransactionsAndMeta>;
    /**
     * Get route info for swapping input token to base and quote tokens required for an LP.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @param liquidityPoolVenue - venue of liquidity pool
     */
    getRouteInfo({ inputTokenAmount, inputTokenMint, lpTokenMint, liquidityPoolVenue }: {
        inputTokenAmount: number;
        inputTokenMint: PublicKey;
        lpTokenMint: PublicKey;
        liquidityPoolVenue?: LiquidityPoolVenues;
    }): Promise<Routes>;
    /**
     * Get token price for a given mint.
     * @param tokenMint - mint of token price to lookup
     * @param tokenDecimals - token decimals
     */
    getJupiterTokenPrice(tokenMint: string, tokenDecimals: number): Promise<number>;
    /**
     * Generates a set of unsigned transactions for swapping a single SPL token to base/quote tokens required for LP.
     * The transactions include setup transactions for the output tokens based on the LP token's base and quote mints.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @param liquidityPoolVenue - venue of liquidity pool
     * @param slippageTolerancePercent - slippage tolerance in percent, between 0 and 100
     * @returns array of unsigned transactions, prices, routes and token infos
     */
    private _swap;
    /**
     * Generates a transaction to add liquidity to a Raydium LP and if required to open the LP token account.
     * @param lpTokenMint - mint of lp token to receive
     * @param poolInfo - Raydium pool info
     * @param tokenAAmount - raw amount of base token to stake in LP
     * @param user - Public key of owner of LP token account
     * @param addLiquiditySlippagePercent - slippage tolerance for adding liquidity in percent, between 0 and 100
     */
    private _createAddLiquidityRaydiumTx;
    /**
     * Generates a transaction to add liquidity to an Orca LP and if required to open the LP token account.
     * @param poolInfo - Orca pool info
     * @param tokenAAmount - raw amount of base token to stake in LP
     * @param tokenBAmount - raw amount of quote token to stake in LP
     * @param user - Public key of owner of LP token account
     * @param addLiquiditySlippagePercent - slippage tolerance for adding liquidity in percent, between 0 and 100
     */
    private _createAddLiquidityOrcaTx;
    /**
     * Get pool info for Raydium LP token mint.
     * @param lpTokenMint - LP token mint
     */
    private _getRaydiumPoolInfo;
    /**
     * Get pool info for Orca LP token mint.
     * @param lpTokenMint - LP token mint
     */
    private _getOrcaPoolInfo;
    /**
     * Get token info for a given mint.
     * @param tokenMint - token mint
     */
    private _getTokenInfo;
}
