"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionBuilder = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const logger_1 = require("./logger");
const anchor_1 = require("@project-serum/anchor");
/**
 * Builder to create a transaction
 */
class TransactionBuilder {
    /**
     * Creates a new transaction builder.
     * @param connection - web3 connection
     * @param owner - owner's public key
     */
    constructor(connection, owner) {
        this._logger = logger_1.Logger.from("sdk/tx-builder");
        this._transaction = new web3_js_1.Transaction();
        this._instructions = [];
        this._connection = connection;
        this._owner = owner;
    }
    /**
     * Adds a token transfer instruction to the transaction if an associated account is not found.
     * @param mint - Public key of the mint
     */
    addCreateTokenAccountIfNotExists(mint) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get associated token address
            const associatedAddress = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, this._owner);
            this._logger.debug('associatedAddress:', associatedAddress);
            // Check if token account exists
            const associatedAccountInfo = yield this._connection.getAccountInfo(associatedAddress);
            // Create token account ix if required
            if (associatedAccountInfo === null) {
                this._logger.debug('associated account not found');
                // Add create token account ix
                var ix = yield spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, associatedAddress, this._owner, this._owner);
                this.add(ix);
                this._logger.debug('generated create associated account ix');
            }
            else {
                this._logger.debug('associated account found');
            }
            return this;
        });
    }
    /**
     * Adds a request compute increase instruction to the transaction.
     * @param units - number of computer units to request, defaults to 256,000.
     */
    addIncreaseComputeInstruction(units = 256000) {
        const data = Buffer.from(Uint8Array.of(0, ...new anchor_1.BN(units).toArray("le", 4)));
        const additionalComputeBudgetInstruction = new web3_js_1.TransactionInstruction({
            keys: [],
            programId: new web3_js_1.PublicKey("ComputeBudget111111111111111111111111111111"),
            data,
        });
        this.add(additionalComputeBudgetInstruction);
        return this;
    }
    /**
     * Saves an instruction to be added to the transaction when built.
     * @param instruction
     */
    add(instruction) {
        // Add instruction
        this._instructions.push(instruction);
        this._logger.debug('number of instructions:', this._instructions.length);
        return this;
    }
    /**
     * Sends and confirms a signed transaction.
     * @param serialisedTx - serialised and signed transaction
     * @returns Transaction signature
     */
    sendAndConfirm(serialisedTx) {
        return __awaiter(this, void 0, void 0, function* () {
            var sig = yield this._connection.sendRawTransaction(serialisedTx);
            yield this._connection.confirmTransaction(sig, 'confirmed');
            return sig;
        });
    }
    /**
     * Clears the instructions and resets the existing transaction.
     */
    clear() {
        this._transaction = new web3_js_1.Transaction();
        this._instructions = [];
        this._logger.warn('cleared transaction builder');
        return this;
    }
    /**
     * Builds a transaction. Adds instructions to the transaction, updates recent blockhash and fee payer.
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            // Add instructions to transaction
            this._transaction.instructions = this._instructions;
            this._logger.debug('number of instructions:', this._transaction.instructions.length);
            // Add recent blockhash
            this._transaction.recentBlockhash = (yield this._connection.getLatestBlockhash()).blockhash;
            this._logger.debug('blockhash:', this._transaction.recentBlockhash);
            // Add fee payer
            this._transaction.feePayer = this._owner;
            this._logger.debug('fee payer:', this._transaction.feePayer);
            return this._transaction;
        });
    }
    /**
     * Builds and signs the transaction.
     * @param wallet - Signing wallet
     * @returns Signed transaction
     */
    buildAndSign(wallet) {
        return __awaiter(this, void 0, void 0, function* () {
            var unsignedTx = yield this.build();
            var signedTx = yield wallet.signTransaction(unsignedTx);
            return signedTx;
        });
    }
    /**
     * Builds, signs, sends and confirms a transaction.
     * @param wallet - Signing wallet
     * @returns Transaction signature
     */
    buildSignSendAndConfirm(wallet) {
        return __awaiter(this, void 0, void 0, function* () {
            var signedTx = yield this.buildAndSign(wallet);
            var sig = yield this.sendAndConfirm(signedTx.serialize());
            return sig;
        });
    }
    /**
     *
     * @param transaction
     * @returns
     */
    rebuild(transaction) {
        return __awaiter(this, void 0, void 0, function* () {
            // Update recent blockhash
            transaction.recentBlockhash = (yield this._connection.getLatestBlockhash()).blockhash;
            this._logger.debug('updated blockhash:', transaction.recentBlockhash);
            // Update fee payer
            transaction.feePayer = this._owner;
            this._logger.debug('updated fee payer:', transaction.feePayer);
            return transaction;
        });
    }
}
exports.TransactionBuilder = TransactionBuilder;
